#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from PyQt5.QtWidgets import QMessageBox

__license__   = 'GPL v3'
__copyright__ = '2019, Jürgen Habelt <juergen@habelt-jena.de>'
__docformat__ = 'restructuredtext en'

load_translations()


if False:
    # This is here to keep my Python error checker from complaining about
    # the built-in functions that will be defined by the plug-in loading system
    # You do not need this code in your plug-ins
    get_icons = get_resources = None


import sys, os, re
from calibre.utils.config import config_dir

# The class that all interface action plug-ins must inherit from
from calibre.gui2.actions import InterfaceAction
from calibre.gui2 import error_dialog
from calibre_plugins.define_content.config import TempConfigWidget
from calibre_plugins.define_content.invoker import Invoker
from calibre_plugins.define_content.logging import LoggingFactory
from calibre_plugins.define_content.json import JsonConfigWrapper 


class DefineContentAction(InterfaceAction):
    '''
    This class performs the work of the plug-in
    '''

    name = u'Define Content'

    # Declare the main action associated with this plug-in
    # The keyboard shortcut can be None if you don't want to use a keyboard shortcut.
    action_spec = (_('Define Content'), None, _('Run the Define Content Action'), None)

    def genesis(self):
        '''
        Initial setup
        The qaction is automatically created from the action_spec defined above
        '''
        self.result_path = os.path.join(config_dir, 'plugins', 'DefineContent', 'result', 'result.html')
        icon = get_icons('images/icon.png')
        self.qaction.setIcon(icon)
        self.qaction.triggered.connect(self.do_action)
        
        JsonConfigWrapper.getInstance()['gui'] = self.gui
        LoggingFactory().getLogger(self).info('Genesis invoked')        # when invoked?
    
    
    def do_action(self):
        '''
        Performs the plug-in action
        '''
        try:
            settings = self.show_dialog()
            if not settings:
                return
            
            invoker = Invoker(settings)
            book_path = self.get_book_path()
            if book_path:
                ecode = invoker.invoke2(book_path, self.result_path) 
                if ecode:
                    error_dialog(
                        self.gui, 
                        _('Bookmark extraction failed'), 
                        _('Cannot extract bookmark information'), 
                        show = True)
                    LoggingFactory().getLogger(self).warning('Cannot extract bookmark information')
                    return
    
                bookmarks = open(self.result_path).read().decode('utf-8')
                self.set_content(bookmarks)
                QMessageBox.information(self.gui, _('Define Content'), _('#content field successfully written'))
                LoggingFactory().getLogger(self).info('#content field successfully written')

        except Exception as ex:
            LoggingFactory().getLogger(self).exception('Exception during command execution.')
            
    
    def apply_settings(self):
        '''
        Prepare the plug-in with new settings from the configuration dialog
        '''
        from calibre_plugins.define_content.config import prefs
        from PyQt5.Qt import QMessageBox
        prefs
        QMessageBox.information(self.gui, _('Define Content'), _('Preferences applied'))


    def get_book_path(self):
        '''
        Get the book path of the first selected e-book
        '''
        rows = self.gui.library_view.selectionModel().selectedRows()
        if not rows or len(rows) == 0:
            error_dialog(
                self.gui, 
                _('No books selected'), 
                _('Cannot add #content as no books are selected'), 
                show = True)
            LoggingFactory().getLogger(self).warning('Cannot add #content as no books are selected')
            return None

        id = self.gui.library_view.model().id(rows[0])          # get the book id
        db = self.gui.current_db.new_api
        formats = db.formats(id)
        if 'EPUB' in formats:
            path = db.format_abspath(id, 'EPUB')
        elif 'PDF' in formats:
            path = db.format_abspath(id, 'PDF')
        else:  
            error_dialog(
                self.gui, 
                _('No required format'), 
                _('Cannot add #content as no pdf or epub format are present: %s' % str(formats)), 
                show = True)
            LoggingFactory().getLogger(self).warning('Cannot add #content as no pdf or epub format are present: %s' % str(formats))
            return None
        
        return path


    def set_content(self, data):
        '''
        Sets the #content metadata of the first selected book
        '''
        sm = self.gui.library_view.selectionModel()         # determine selected book
        row = sm.selectedRows()[0]
        id = self.gui.library_view.model().id(row)
        
        db = self.gui.current_db.new_api                    # update its #content custom field
        md = db.get_metadata(id)
        md.__setattr__(u'#content', data)
        db.set_metadata(id, md)
        
        self.gui.library_view.model().refresh()             # refresh the model
        sm.select(row, sm.Rows | sm.SelectCurrent)          # (re-select to update the details view)

    
    def show_dialog(self):
        '''
        This dialog is for temporary configuration
        '''
        dlg = TempConfigWidget(self.gui)
        if dlg.exec_():
            return dlg.layout.prefs
        
        return None
