#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

__license__   = 'GPL v3'
__copyright__ = u'2019, Jürgen Habelt <juergen@habelt-jena.de>'
__docformat__ = 'restructuredtext en'

load_translations()


from time import sleep
from os.path import expanduser
from PyQt5.Qt import Qt, QFileDialog, QWidget, QDialog, QGridLayout, QMessageBox, QPushButton, \
     QLabel, QLineEdit, QSpinBox, QCheckBox, QFrame, QObject, QHeaderView, QSize, QResizeEvent, QCoreApplication

from calibre_plugins.define_content.logging import LoggingFactory
from calibre_plugins.define_content.json import JsonConfigWrapper 
from calibre_plugins.define_content.model import RequirementsModel, RequirementsDelegate
from calibre_plugins.define_content.gui.RequirementsView import Ui_requirementsView
 

# This is where all preferences for this plug-in will be stored
prefs = JsonConfigWrapper.getInstance()


class ConfigLayout(QGridLayout):
    '''
    Shall display the Define Content configuration
    '''
    
    def __init__(self, widget, prefs):
        '''
        Initializes the Layout with the required Widgets 
        '''
        QGridLayout.__init__(self)
        self.setColumnStretch(0, 0)
        self.setColumnStretch(1, 0)
        self.setRowStretch(0, 0)
        self.setRowStretch(1, 0)
        self.setRowStretch(2, 0)
        self.setRowStretch(3, 0)
        
        self.widget = widget
        self.prefs = prefs
        self.levelLabel = QLabel(_('Max Level:'))
        self.addWidget(self.levelLabel, 0, 0)
        
        self.level = QSpinBox()
        self.level.setValue(prefs['max_level'])
        self.level.setToolTip(_('The maximum level of Headings to be displayed in #content'))
        self.addWidget(self.level, 0, 1)
        self.levelLabel.setBuddy(self.level)
        
        self.stripLabel = QLabel(_('Strip:'))
        self.addWidget(self.stripLabel, 1, 0)
        
        self.useStrip = QCheckBox(self.widget)
        self.useStrip.setChecked(prefs['use_strip'])
        self.useStrip.setToolTip(_('A flag indicating whether to strip some characters at the beginning of Headings'))
        self.addWidget(self.useStrip, 1, 1)
        
        self.strip = QLineEdit(self.widget)
        self.strip.setText(prefs['strip'])
        self.strip.setToolTip(_('A regular expression defining the characters to be stripped'))
        self.addWidget(self.strip, 1, 2)
        self.stripLabel.setBuddy(self.strip)
        
        self.omitLabel = QLabel(_('Omit:'))
        self.addWidget(self.omitLabel, 2, 0)
        
        self.omit = QCheckBox(self.widget)
        self.omit.setChecked(prefs['omit'])
        self.omit.setToolTip(_('A flag indicating whether to omit the list item numbers of second level Headings'))
        self.addWidget(self.omit, 2, 1)
        self.omitLabel.setBuddy(self.omit)
        
        self.continuousLabel = QLabel(_('Continuous:'))
        self.addWidget(self.continuousLabel, 3, 0)
        
        self.continuous = QCheckBox(self.widget)
        self.continuous.setChecked(prefs['continuous'])
        self.continuous.setToolTip(_('A flag indicating whether to count second level Headings continuously'))
        self.addWidget(self.continuous, 3, 1)
        self.continuousLabel.setBuddy(self.continuous)
        
        self.row_count = 4
        
        
    def save_settings(self):
        '''
        Saves the settings
        '''
        self.prefs['max_level'] = self.level.value()
        self.prefs['use_strip'] = self.useStrip.isChecked()
        self.prefs['strip'] = self.strip.text()
        self.prefs['omit'] = self.omit.isChecked()
        self.prefs['continuous'] = self.continuous.isChecked()

    
class ConfigWidget(QWidget):
    '''
    The Configuration widget for permanent settings
    '''

    def __init__(self):
        '''
        Initialization, constructs the widget.
        '''
        QWidget.__init__(self)

        self.layout = ConfigLayout(self, prefs)
        self.setLayout(self.layout)
        self.layout.setRowStretch(self.layout.row_count, 0)
        self.layout.setRowStretch(self.layout.row_count + 1, 0)

        self.divider = QFrame()
        self.divider.setFrameShape(QFrame.HLine)
        self.divider.setFrameShadow(QFrame.Sunken)        
        self.layout.addWidget(self.divider, self.layout.row_count, 0, 1, 3)

        self.pdftk_label = QLabel(_('Pdftk Tool:'))
        self.layout.addWidget(self.pdftk_label, self.layout.row_count + 1, 0)
        
        self.pdftk_line_edit = QLineEdit(self)
        self.pdftk_line_edit.setText(prefs['pdftk'])
        self.pdftk_line_edit.setToolTip(_('Defines the path of the Pdftk tool'))
        self.layout.addWidget(self.pdftk_line_edit, self.layout.row_count + 1, 2)
        self.pdftk_label.setBuddy(self.pdftk_line_edit)
        
        self.pdftk_button = QPushButton(_('Select'), self)
        self.pdftk_button.clicked.connect(self.select)
        self.pdftk_button.setToolTip(_('Show a file dialog to select the Pdftk tool'))
        self.layout.addWidget(self.pdftk_button, self.layout.row_count + 1, 1)

        self.log_label = QLabel(_('Log File:'))
        self.layout.addWidget(self.log_label, self.layout.row_count + 2, 0)
        
        self.log_line_edit = QLineEdit(self)
        self.log_line_edit.setText(prefs['log_path'])
        self.log_line_edit.setToolTip(_('Defines the path of the Ruby invocation log file'))
        self.layout.addWidget(self.log_line_edit, self.layout.row_count + 2, 2)
        self.log_label.setBuddy(self.log_line_edit)
        
        self.log_button = QPushButton(_('Select'), self)
        self.log_button.clicked.connect(self.select_log)
        self.log_button.setToolTip(_('Show a file dialog to select the Ruby log file'))
        self.layout.addWidget(self.log_button, self.layout.row_count + 2, 1)

        self.ruby_label = QLabel(_('Ruby Name:'))
        self.layout.addWidget(self.ruby_label, self.layout.row_count + 3, 0)
        
        self.ruby_line_edit = QLineEdit(self)
        self.ruby_line_edit.setText(prefs['ruby'])
        self.ruby_line_edit.setToolTip(_('Defines the Ruby executable name'))
        self.layout.addWidget(self.ruby_line_edit, self.layout.row_count + 3, 2)
        self.ruby_label.setBuddy(self.ruby_line_edit)
        
        self.about_button = QPushButton(_('About...'), self)
        self.about_button.clicked.connect(self.about)
        self.about_button.setToolTip(_('Displays an About box for the plug-in'))
        self.layout.addWidget(self.about_button, self.layout.row_count + 4, 1)

        self.help_label = QLabel(_('<a href="https://juergen.habelt-jena.de/en/programming-projects/define-content/application?start=1">Application Hints</a>'), self)
        self.help_label.setToolTip(_('Invokes a page on the Internet with application hints'))
        self.help_label.setTextInteractionFlags(Qt.LinksAccessibleByMouse | Qt.TextBrowserInteraction)
        self.help_label.setOpenExternalLinks(True)
        self.layout.addWidget(self.help_label, self.layout.row_count + 4, 2)
        
        self.requirements_button = QPushButton(_('Check...'), self)
        self.requirements_button.clicked.connect(self.requirements)
        self.requirements_button.setToolTip(_('Displays a Dialog with requirements status'))
        self.layout.addWidget(self.requirements_button, self.layout.row_count + 5, 1)
        
        
    def select(self):
        '''
        Displays a file dialog to select the Pdftk tool
        '''
        res = QFileDialog.getOpenFileName(self, _('Select PDFTK Tool'))[0]
        if res:
            self.pdftk_line_edit.setText(res)
        
        
    def select_log(self):
        '''
        Displays a file dialog to select the Ruby log file
        '''
        res = QFileDialog.getSaveFileName(self, _('Select Ruby Log File'), prefs.get_path())[0]
        if res:
            self.log_line_edit.setText(res)


    def about(self):
        '''
        Get the about text from a file inside the plug-in zip file
        '''
        text = get_resources('about.txt')
        QMessageBox.about(self, _('About the Define Content Plug-in'), text.decode('utf-8'))


    def requirements(self):
        '''
        Shows the Requirements status of the plug-in
        '''
        try:
            self.requirements = RequirementsView() 
            self.requirements.show()            
            self.requirements.model.updateConfigurationState()
            
        except Exception as ex:
            LoggingFactory().getLogger(self).exception("Exception in Requirements Dialog")
        
        
    def save_settings(self):
        '''
        Saves the settings
        '''
        self.layout.save_settings()
        prefs['pdftk'] = self.pdftk_line_edit.text()
        prefs['log_path'] = self.log_line_edit.text()
        prefs['ruby'] = self.ruby_line_edit.text()


class TempConfigWidget(QDialog):
    '''
    For temporary configuration during action
    '''
    
    def __init__(self, gui):
        '''
        Initialization, constructs the widget.
        '''
        QDialog.__init__(self, gui)
        
        self.setWindowTitle(_('Temporary Settings'))

        self.layout = ConfigLayout(self, prefs.dup())
        self.setLayout(self.layout)
        
        self.button = QPushButton(_('Proceed'), self)
        self.button.clicked.connect(self.proceed)
        self.button.setToolTip(_('Proceed with the Define Content action'))
        self.layout.addWidget(self.button, 4, 2)
        
        
    def proceed(self):
        '''
        Continuous the action        
        '''
        self.layout.save_settings()
        self.accept()
       

class RequirementsView(QDialog, Ui_requirementsView): 
    '''
    Dialog class using the UI generated python class Ui_RequirementsView
    '''

    def __init__(self):
        '''
        Initializes the Dialog with the UI
        '''
        super(RequirementsView, self).__init__()
        self.setupUi(self)
        self.setWindowTitle(_('Requirements'))
        
        self.model = RequirementsModel(self.tableView)
        self.tableView.setModel(self.model)
        self.delegate = RequirementsDelegate(self)
        self.tableView.setItemDelegate(self.delegate)

        self.tableView.horizontalHeader().setSectionResizeMode(1, QHeaderView.ResizeToContents)
        self.tableView.horizontalHeader().setSectionResizeMode(0, QHeaderView.Stretch)

        